/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2003 Hiroyuki Ikezoe
 *  Copyright (C) 2003 - 2004 Takuro Ashie
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n.h>

#include "kazehakase.h"
#include "kz-root-bookmark.h"
#include "kz-bookmark-bar.h"
#include "kz-bookmark-menu.h"
#include "kz-bookmark-item.h"
#include "kz-bookmark.h"
#include "kz-bookmark-file.h"
#include "kz-bookmark-separator.h"
#include "kz-bookmark-utils.h"
#include "kz-actions.h"
#include "kz-notebook.h"
#include "utils.h"
#include "glib-utils.h"

typedef struct _KzBookmarkBarPrivate	KzBookmarkBarPrivate;
struct _KzBookmarkBarPrivate
{
	GtkWidget *dragged_item;
};
#define KZ_BOOKMARK_BAR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_BOOKMARK_BAR, KzBookmarkBarPrivate))

enum {
	PROP_0,
	PROP_KZ_WINDOW,
	PROP_BOOKMARK_LIST
};

enum {
	TARGET_GTK_NOTEBOOK_TAB,
	TARGET_KAZEHAKASE_BOOKMARKS,
	TARGET_NETSCAPE_URL,
	TARGET_TEXT_URI_LIST,
	TARGET_TEXT_PLAIN,
	TARGET_STRING
};

static const GtkTargetEntry url_drag_types [] = 
{
	{"GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, TARGET_GTK_NOTEBOOK_TAB},
	{"_KAZEHAKASE_BOOKMARKS", GTK_TARGET_SAME_APP, TARGET_KAZEHAKASE_BOOKMARKS},
        { "_NETSCAPE_URL",        0, TARGET_NETSCAPE_URL},
	{ "text/uri-list",        0, TARGET_TEXT_URI_LIST},
	{ "text/plain",      0, TARGET_TEXT_PLAIN},
	{ "STRING",          0, TARGET_STRING}
};

static const guint n_url_drag_types = G_N_ELEMENTS (url_drag_types);

static GObject* constructor  (GType           type,
			      guint           n_props,
			      GObjectConstructParam *props);
static void     dispose      (GObject *object);
static void     set_property (GObject       *object,
			      guint          prop_id,
			      const GValue  *value,
			      GParamSpec    *pspec);
static void     get_property (GObject       *object,
			      guint          prop_id,
			      GValue        *value,
			      GParamSpec    *pspec);

static gboolean button_release (GtkWidget      *widget,
			        GdkEventButton *event);
static void     drag_data_received
			     (GtkWidget *widget,
			      GdkDragContext *context,
			      gint x, gint y,
			      GtkSelectionData *data,
			      guint info,
			      guint time);
static void     drag_leave   (GtkWidget *widget,
			      GdkDragContext *context,
			      guint time);
static gboolean drag_motion  (GtkWidget *widget,
			      GdkDragContext *context,
			      gint x, gint y,
			      guint time);

static void     kz_bookmark_bar_refresh_all  (KzBookmarkBar *bar);

static void     kz_bookmark_bar_connect_signal    (KzBookmarkBar *bar);
static void     kz_bookmark_bar_disconnect_signal (KzBookmarkBar *bar);


static void     cb_bookmark_list_updated         (KzBookmark *folder,
						  KzBookmarkBar *bar);
static void     cb_bookmark_list_remove_child    (KzBookmark *folder,
						  KzBookmark *child,
						  KzBookmarkBar *bar);
static void     cb_bookmark_list_insert_child  	 (KzBookmark *folder,
						  KzBookmark *child,
						  KzBookmark *sibling,
						  KzBookmarkBar *bar);

static GtkToolItem *create_tool_item (KzBookmarkBar *bar,
				      KzBookmark *child);

G_DEFINE_TYPE(KzBookmarkBar, kz_bookmark_bar, GTK_TYPE_EVENT_BOX)

static void
kz_bookmark_bar_class_init (KzBookmarkBarClass *klass)
{
	GObjectClass *gobject_class;
	GtkWidgetClass *widget_class;

	gobject_class = G_OBJECT_CLASS(klass);
	widget_class  = GTK_WIDGET_CLASS(klass);

	/* GObject class */
	gobject_class->constructor  = constructor;
	gobject_class->dispose      = dispose;
	gobject_class->set_property = set_property;
	gobject_class->get_property = get_property;

	widget_class->button_release_event = button_release;
	widget_class->drag_data_received   = drag_data_received;
	widget_class->drag_leave           = drag_leave;
	widget_class->drag_motion          = drag_motion;

	g_object_class_install_property
		(gobject_class,
		 PROP_KZ_WINDOW,
		 g_param_spec_object ("kz-window",
				      _("KzWindow"),
				      _("The parent kazehakase window"),
				      KZ_TYPE_WINDOW,
				      G_PARAM_READWRITE |
				      G_PARAM_CONSTRUCT_ONLY));

	g_object_class_install_property
		(gobject_class,
		 PROP_BOOKMARK_LIST,
		 g_param_spec_object ("bookmark-folder",
				      _("BookmarkFolder"),
				      _("Bookmarks list to show"),
				      KZ_TYPE_BOOKMARK,
				      G_PARAM_READWRITE |
				      G_PARAM_CONSTRUCT_ONLY));
	g_type_class_add_private(gobject_class, sizeof(KzBookmarkBarPrivate));
}


static void
kz_bookmark_bar_init (KzBookmarkBar *bar)
{
	KzBookmarkBarPrivate *priv = KZ_BOOKMARK_BAR_GET_PRIVATE (bar);
	bar->toolbar = gtk_toolbar_new();
	bar->kz      = NULL;
	bar->folder  = NULL;
	
	priv->dragged_item = GTK_WIDGET(gtk_separator_tool_item_new());
	g_object_ref(priv->dragged_item);

	gtk_container_add(GTK_CONTAINER(bar), bar->toolbar);
	gtk_widget_show(bar->toolbar);

	gtk_drag_dest_set(GTK_WIDGET(bar), 
			  GTK_DEST_DEFAULT_ALL, 
                          url_drag_types, n_url_drag_types,
			  GDK_ACTION_LINK | GDK_ACTION_MOVE);
}


static void
dispose (GObject *object)
{
	KzBookmarkBar *bar = KZ_BOOKMARK_BAR(object);
	KzBookmarkBarPrivate *priv = KZ_BOOKMARK_BAR_GET_PRIVATE (bar);

	if (bar->folder)
	{
		kz_bookmark_bar_disconnect_signal(bar);
		g_object_unref(bar->folder);
		bar->folder = NULL;
	}

	if (bar->kz)
	{
		g_object_unref(bar->kz);
		bar->kz = NULL;
	}

	if (priv->dragged_item)
	{
		gtk_widget_destroy(GTK_WIDGET(priv->dragged_item));
		g_object_unref(priv->dragged_item);
		priv->dragged_item = NULL;
	}

	if (G_OBJECT_CLASS (kz_bookmark_bar_parent_class)->dispose)
		G_OBJECT_CLASS (kz_bookmark_bar_parent_class)->dispose(object);
}


static GObject*
constructor (GType                  type,
             guint                  n_props,
             GObjectConstructParam *props)
{
	KzBookmarkBar *bar;

	GObject *object;
	GObjectClass *klass = G_OBJECT_CLASS(kz_bookmark_bar_parent_class);

	object = klass->constructor(type, n_props, props);

	bar = KZ_BOOKMARK_BAR(object);

	kz_bookmark_bar_refresh_all(bar);
	return object;
}


static void
set_property (GObject         *object,
              guint            prop_id,
              const GValue    *value,
              GParamSpec      *pspec)
{
	KzBookmarkBar *bar = KZ_BOOKMARK_BAR(object);
  
	switch (prop_id)
	{
	case PROP_KZ_WINDOW:
		bar->kz = g_value_dup_object(value);
		break;
	case PROP_BOOKMARK_LIST:
		bar->folder = g_value_dup_object(value);
		kz_bookmark_bar_connect_signal(bar);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}


static void
get_property (GObject         *object,
              guint            prop_id,
              GValue          *value,
              GParamSpec      *pspec)
{
	KzBookmarkBar *bar = KZ_BOOKMARK_BAR(object);

	switch (prop_id)
	{
	case PROP_KZ_WINDOW:
		g_value_set_object(value, bar->kz);
		break;
	case PROP_BOOKMARK_LIST:
		g_value_set_object(value, bar->folder);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}


static gboolean
button_release (GtkWidget *widget, GdkEventButton *event)
{
	KzBookmarkBar *bar;

	bar = KZ_BOOKMARK_BAR(widget);

	switch (event->button)
	{
	case 1:
		break;
	case 2:
		break;
	case 3:
		kz_actions_popup_bookmark_menu_modal(bar->kz,
						     bar->folder,
						     event->button,
						     event->time);
		return TRUE;
		break;
	default:
		break;
	}
	return FALSE;
}


GtkWidget *
kz_bookmark_bar_new (KzWindow *kz, KzBookmark *folder)
{
	GObject *obj;

	g_return_val_if_fail(KZ_IS_BOOKMARK(folder), NULL);
	g_return_val_if_fail(kz_bookmark_is_folder(folder), NULL);

	obj = g_object_new(KZ_TYPE_BOOKMARK_BAR,
			   "kz-window",       kz,
			   "bookmark-folder", folder,
			   NULL);

	return GTK_WIDGET(obj);
}


static void
remove_item (GtkWidget *child, gpointer data)
{
	gtk_container_remove(GTK_CONTAINER(child->parent), child);
}

static void
insert_tool_item (KzBookmark *bookmark, KzBookmarkBar *bar)
{
	GtkToolItem *toolitem;
	
	toolitem = create_tool_item(bar, bookmark);
	gtk_toolbar_insert(GTK_TOOLBAR(bar->toolbar),
			   toolitem, -1);
}

static void
kz_bookmark_bar_refresh_all (KzBookmarkBar *bar)
{
	gtk_container_foreach (GTK_CONTAINER (GTK_TOOLBAR(bar->toolbar)),
			       remove_item, NULL);

	kz_bookmark_folder_foreach_child(KZ_BOOKMARK_FOLDER(bar->folder),
					 (GFunc)insert_tool_item, bar);

	gtk_widget_queue_resize(GTK_WIDGET(bar));
}


static void
kz_bookmark_bar_connect_signal (KzBookmarkBar *bar)
{
	g_signal_connect(bar->folder, "children-reordered",
			 G_CALLBACK(cb_bookmark_list_updated),
			 bar);	
	g_signal_connect(bar->folder, "insert-child",
			 G_CALLBACK(cb_bookmark_list_insert_child),
			 bar);
	g_signal_connect(bar->folder, "remove-child",
			 G_CALLBACK(cb_bookmark_list_remove_child),
			 bar);	
}


static void
kz_bookmark_bar_disconnect_signal (KzBookmarkBar *bar)
{
	g_signal_handlers_disconnect_by_func
		(bar->folder,
		 G_CALLBACK(cb_bookmark_list_updated), bar);
	g_signal_handlers_disconnect_by_func
		(bar->folder,
		 G_CALLBACK(cb_bookmark_list_insert_child), bar);
	g_signal_handlers_disconnect_by_func
		(bar->folder,
		 G_CALLBACK(cb_bookmark_list_remove_child), bar);
}


static void
drag_data_received (GtkWidget *widget,
                    GdkDragContext *context,
                    gint x, gint y,
                    GtkSelectionData *data,
                    guint info,
                    guint time)
{
	KzBookmarkBar *bar;
	KzBookmark *bookmark = NULL, *sibling;
        KzBookmarkFolder *parent;
	GtkWidget *src_widget;
	gint index;
	gboolean moved = FALSE, success = FALSE;

	bar = KZ_BOOKMARK_BAR(widget);
	index = gtk_toolbar_get_drop_index(GTK_TOOLBAR(bar->toolbar),
					   x, y);

	sibling = kz_bookmark_folder_get_nth_child(KZ_BOOKMARK_FOLDER(bar->folder), index);

	switch (info)
	{
	 case TARGET_GTK_NOTEBOOK_TAB:
	 {
	 	KzWeb *kzweb;
		const gchar *uri, *title;
		src_widget = gtk_drag_get_source_widget(context);
		if (!KZ_IS_NOTEBOOK(src_widget))
			break;
		kzweb = KZ_WEB(kz_notebook_get_dragged_page(KZ_NOTEBOOK(src_widget)));
		uri   = kz_web_get_location(kzweb);
		title = kz_web_get_title(kzweb);
		bookmark = KZ_BOOKMARK(kz_bookmark_new_with_attrs(title, uri, NULL));
	 	break;
	 }
	 case TARGET_KAZEHAKASE_BOOKMARKS:
		src_widget = gtk_drag_get_source_widget(context);
		
		if (!KZ_IS_BOOKMARK_ITEM(src_widget))
			break;

		bookmark = KZ_BOOKMARK_ITEM(src_widget)->bookmark;
		if (bookmark == sibling)
		{
			bookmark = NULL;
			break;
		}
	
		g_object_ref(bookmark);
		parent = kz_bookmark_get_parent(bookmark);
		kz_bookmark_folder_remove(parent, bookmark);
		moved = TRUE;
	 	break;
	 case TARGET_NETSCAPE_URL:
	 case TARGET_TEXT_URI_LIST:
	 	bookmark = kz_bookmark_create_from_selection_data(data);
		break;
	 default:
		break;
	}

	if (bookmark)
	{
		if (sibling)
		{
			kz_bookmark_folder_insert_before(KZ_BOOKMARK_FOLDER(bar->folder),
						  bookmark,
						  sibling);
		}
		else
		{
			kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(bar->folder), bookmark);
		}
		g_object_unref(bookmark);
		success = TRUE;
	}

	gtk_drag_finish(context, success, moved, time);
}

static void
drag_leave   (GtkWidget *widget,
              GdkDragContext *context,
              guint time)
{
	KzBookmarkBar *bar;

	bar = KZ_BOOKMARK_BAR(widget);
	gtk_toolbar_set_drop_highlight_item(GTK_TOOLBAR(bar->toolbar),
					    NULL, 0);
}


static gboolean 
drag_motion (GtkWidget *widget,
             GdkDragContext *context,
             gint x, gint y,
             guint time)
{
	KzBookmarkBar *bar;
	KzBookmarkBarPrivate *priv = KZ_BOOKMARK_BAR_GET_PRIVATE (widget);
	gint index;

	bar = KZ_BOOKMARK_BAR(widget);
	index = gtk_toolbar_get_drop_index(GTK_TOOLBAR(bar->toolbar),
					   x, y);

	gtk_toolbar_set_drop_highlight_item(GTK_TOOLBAR(bar->toolbar),
					    GTK_TOOL_ITEM(priv->dragged_item), index);
	
	gdk_drag_status(context, context->suggested_action, time);

	return TRUE;
}


static GtkToolItem *
create_tool_item (KzBookmarkBar *bar, KzBookmark *child)
{
	GtkToolItem *toolitem;
	
	g_return_val_if_fail(KZ_IS_BOOKMARK(child), NULL);
	g_return_val_if_fail(KZ_IS_BOOKMARK_BAR(bar), NULL);

	if (kz_bookmark_is_separator(child))
		toolitem = gtk_separator_tool_item_new();
	else
		toolitem = kz_bookmark_item_new(bar->kz, child);
	
	gtk_widget_show(GTK_WIDGET(toolitem));
	return toolitem;	
}


static void
cb_bookmark_list_updated (KzBookmark *folder, KzBookmarkBar *bar)
{
	g_return_if_fail(KZ_IS_BOOKMARK_BAR(bar));

	kz_bookmark_bar_refresh_all(bar);
}


static void
cb_bookmark_list_insert_child  (KzBookmark *folder,
				KzBookmark *child,
				KzBookmark *sibling,
				KzBookmarkBar *bar)
{
	gint index = -1;
	GtkToolItem *toolitem;
	
	g_return_if_fail(KZ_IS_BOOKMARK(child));
	g_return_if_fail(KZ_IS_BOOKMARK_BAR(bar));

	if (sibling)
		index = kz_bookmark_folder_get_child_index(KZ_BOOKMARK_FOLDER(folder), sibling);

	toolitem = create_tool_item(bar, child);
	
	gtk_toolbar_insert(GTK_TOOLBAR(bar->toolbar),
			   toolitem, index);
}


static void
cb_bookmark_list_remove_child  (KzBookmark *folder,
				KzBookmark *child,
				KzBookmarkBar *bar)
{
	gint index;
	GtkToolItem *toolitem;

	g_return_if_fail(KZ_IS_BOOKMARK(child));
	g_return_if_fail(KZ_IS_BOOKMARK_BAR(bar));

	index = kz_bookmark_folder_get_child_index(KZ_BOOKMARK_FOLDER(folder), child);
	if (index < 0) return;
	
	toolitem = gtk_toolbar_get_nth_item(GTK_TOOLBAR(bar->toolbar),
					    index);
	if (!toolitem) return;
	gtk_widget_destroy(GTK_WIDGET(toolitem));
}

